@@ -1,5 +1,5 @@
 #include <QAbstractItemView>
-/* Copyright (C) 2006-2022 J.F.Dockes
+/* Copyright (C) 2006-2025 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -58,7 +58,43 @@
 #include "scbase.h"
 #include "base64.h"
 
-using namespace std;
+// If text does not end with space, return last (partial) word and its start offset (>=0) else
+// return -1
+static int getPartialWord(const QString& txt, QString& word)
+{
+    // Extract last word in text
+    if (txt.isEmpty()) {
+        return -1;
+    }
+    int lstidx = txt.size()-1;
+
+    // If the input ends with a space or dquote (phrase input), or
+    // dquote+qualifiers, no partial word.
+    if (txt[lstidx] == ' ') {
+        return -1;
+    }
+    int cs = txt.lastIndexOf("\"");
+    if (cs > 0) {
+        bool dquoteToEndNoSpace{true};
+        for (int i = cs; i <= lstidx; i++) {
+            if (txt[i] == ' ') {
+                dquoteToEndNoSpace = false;
+                break;
+            }
+        }
+        if (dquoteToEndNoSpace) {
+            return -1;
+        }
+    }
+
+    cs = txt.lastIndexOf(" ");
+    if (cs < 0)
+        cs = 0;
+    else
+        cs++;
+    word = txt.right(txt.size() - cs);
+    return cs;
+}
 
 // Max db term matches fetched from the index
 static const int maxdbtermmatch = 20;
@@ -76,7 +112,7 @@ void RclCompleterModel::init()
 int RclCompleterModel::rowCount(const QModelIndex &) const
 {
     LOGDEB1("RclCompleterModel::rowCount: " << currentlist.size() << "\n");
-    return currentlist.size();
+    return static_cast<int>(currentlist.size());
 }
 
 int RclCompleterModel::columnCount(const QModelIndex &) const
@@ -110,9 +146,11 @@ QVariant RclCompleterModel::data(const QModelIndex &index, int role) const
     return QVariant();
 }
 
+// Compute possible completion for current search state. Entering a space first requests a history
+// list. Otherwise, list a partial word against history and index and fill the model
 void RclCompleterModel::onPartialWord(int tp, const QString& _qtext, const QString& qpartial)
 {
-    string partial = qs2u8s(qpartial);
+    std::string partial = qs2u8s(qpartial);
     QString qtext = _qtext.trimmed();
     bool onlyspace = qtext.isEmpty();
     LOGDEB1("RclCompleterModel::onPartialWord: [" << partial << "] onlyspace "<< onlyspace << "\n");
@@ -121,8 +159,7 @@ void RclCompleterModel::onPartialWord(int tp, const QString& _qtext, const QStri
     beginResetModel();
     if ((prefs.ssearchNoComplete && !onlyspace) || tp == SSearch::SST_FNM) {
         // Nocomplete: only look at history by entering space
-        // Filename: no completion for now. We'd need to termatch with
-        // the right prefix?
+        // Filename: no completion for now. We'd need to termatch with the right prefix?
         endResetModel();
         return;
     }
@@ -141,22 +178,22 @@ void RclCompleterModel::onPartialWord(int tp, const QString& _qtext, const QStri
             currentlist.push_back({prefs.ssearchHistory[i], -1});
         }
     }
-    firstfromindex = currentlist.size();
+    firstfromindex = static_cast<int>(currentlist.size());
 
     // Look for Recoll terms beginning with the partial word. If the index is not stripped, only do
     // this after the partial has at least 2 characters, else the syn/diac/case expansion is too
     // expensive
     int mintermsizeforexpand = o_index_stripchars ? 1 : 2;
-    if (qpartial.trimmed().size() >= mintermsizeforexpand) {
+    if (qpartial.trimmed().size() >= mintermsizeforexpand &&
+        partial.find_first_of("[?*") == std::string::npos) {
         Rcl::TermMatchResult rclmatches;
-        if (!rcldb->termMatch(Rcl::Db::ET_WILD, string(),
-                              partial + "*", rclmatches, maxdbtermmatch)) {
+        if (!rcldb->termMatch(Rcl::Db::ET_WILD, "", partial + "*", rclmatches, maxdbtermmatch)) {
             LOGDEB1("RclCompleterModel: termMatch failed: [" << partial + "*" << "]\n");
         } else {
-            LOGDEB1("RclCompleterModel: termMatch cnt: " << rclmatches.entries.size() << endl);
+            LOGDEB1("RclCompleterModel: termMatch cnt: " << rclmatches.entries.size() << '\n');
         }
         for (const auto& entry : rclmatches.entries) {
-            LOGDEB1("RclCompleterModel: match " << entry.term << endl);
+            LOGDEB1("RclCompleterModel: match " << entry.term << '\n');
             currentlist.push_back({u8s2qs(entry.term), entry.wcf});
         }
     }
@@ -189,6 +226,9 @@ void SSearch::init()
     popup->horizontalHeader()->hide();
     popup->verticalHeader()->hide();
     m_completer->setPopup(popup);
+    // We need unfilteredPopupCompletion, else the completer does not work for completing the last
+    // word of a multiword entry (filters against the whole line contents). Tried
+    // setCompletionPrefix() with no success.
     m_completer->setCompletionMode(QCompleter::UnfilteredPopupCompletion);
     m_completer->setFilterMode(Qt::MatchContains);
     m_completer->setCaseSensitivity(Qt::CaseInsensitive);
@@ -197,16 +237,25 @@ void SSearch::init()
     m_completer->popup()->installEventFilter(this);
     queryText->installEventFilter(this);
     connect(this, SIGNAL(partialWord(int, const QString&, const QString&)),
-            m_completermodel,
-            SLOT(onPartialWord(int,const QString&,const QString&)));
+            m_completermodel, SLOT(onPartialWord(int,const QString&,const QString&)));
     connect(m_completer, SIGNAL(activated(const QString&)), this,
             SLOT(onCompletionActivated(const QString&)));
+
     connect(historyPB, SIGNAL(clicked()), this, SLOT(onHistoryClicked()));
+
+    int searchtype = prefs.ssearchTypSav ? prefs.ssearchTyp : SSearch::SST_LANG;
+    onSearchTypeChanged(searchtype);
+    
     setupButtons();
     onNewShortcuts();
     connect(&SCBase::scBase(), SIGNAL(shortcutsChanged()),this, SLOT(onNewShortcuts()));
 }
 
+SSearch::SSearchType SSearch::getSearchType()
+{
+    return (SSearchType)searchTypCMB->currentIndex();
+}
+
 void SSearch::onNewShortcuts()
 {
     SETSHORTCUT(this, "ssearch:197", tr("Simple search"), tr("History"),
@@ -252,7 +301,6 @@ void SSearch::clearAll()
 
 void SSearch::onCompleterShown()
 {
-    LOGDEB("SSearch::onCompleterShown\n");
     QCompleter *completer = queryText->completer();
     if (!completer) {
         LOGDEB0("SSearch::onCompleterShown: no completer\n");
@@ -278,26 +326,23 @@ void SSearch::onCompleterShown()
         return;
     }
     
-    LOGDEB0("SSearch::onCompleterShown:" <<  
-            " current [" << qs2utf8s(currentText()) <<
-            "] saved [" << qs2utf8s(m_savedEditText) <<
-            "] popup [" << qs2utf8s(text) << "]\n");
+    LOGDEB0("SSearch::onCompleterShown:" <<  " current [" << qs2utf8s(currentText()) <<
+            "] saved [" << qs2utf8s(m_savedEditText) << "] popup [" << qs2utf8s(text) << "]\n");
 
-    // We append the completion part to the end of the current input,
-    // line, and select it so that the user has a clear indication of
-    // what will happen if they type Enter.
-    int pos = queryText->cursorPosition();
-    int len = text.size() - currentText().size();
-    queryText->setText(text);
-    queryText->setCursorPosition(pos);
-    queryText->setSelection(pos, len);
+    if (!prefs.ssearchCompletePassive) {
+        // We append the completion part to the end of the current input, line, and select it so
+        // that the user has a clear indication of what will happen if they type Enter.
+        int pos = queryText->cursorPosition();
+        int len = text.size() - currentText().size();
+        queryText->setText(text);
+        queryText->setCursorPosition(pos);
+        queryText->setSelection(pos, len);
+    }
 }
 
-// This is to avoid that if the user types Backspace or Del while we
-// have inserted / selected the current completion, the lineedit text
-// goes back to what it was, the completion fires, and it looks like
-// nothing was typed. Disable the completionn after Del or Backspace
-// is typed.
+// This is to avoid that if the user types Backspace or Del while we have inserted / selected the
+// current completion, the lineedit text goes back to what it was, the completion fires, and it
+// looks like nothing was typed. Disable the completion after Del or Backspace is typed.
 bool SSearch::eventFilter(QObject *target, QEvent *event)
 {
     Q_UNUSED(target);
@@ -305,41 +350,42 @@ bool SSearch::eventFilter(QObject *target, QEvent *event)
     if (event->type() != QEvent::KeyPress) {
         return false;
     }
-    LOGDEB1("SSearch::eventFilter: KeyPress event. Target " << target <<
-            " popup "<<m_completer->popup() << " lineedit "<<queryText<< "\n");
-
     QKeyEvent *keyEvent = (QKeyEvent *)event;
-    if (keyEvent->key() == Qt::Key_Backspace ||
-        keyEvent->key()==Qt::Key_Delete) {
+    LOGDEB1("SSearch::eventFilter: KeyPress event: " << keyEvent->key() << " Target " << target <<
+            " popup "<<m_completer->popup() << " lineedit "<<queryText<< "\n");
+    if (keyEvent->key() == Qt::Key_Backspace || keyEvent->key()==Qt::Key_Delete) {
         LOGDEB("SSearch::eventFilter: backspace/delete\n");
         queryText->setCompleter(nullptr);
         return false;
-    } else {
-        if (nullptr == queryText->completer()) {
-            queryText->setCompleter(m_completer);
+    } else if (keyEvent->key() == Qt::Key_Enter || keyEvent->key() == Qt::Key_Return) {
+        if (prefs.ssearchCompletePassive &&
+            queryText->completer() && queryText->completer()->popup() &&
+            queryText->completer()->popup()->isVisible()) {
+            queryText->completer()->popup()->hide();
+            restoreText(true);
+            return true;
         }
-    }        
+    } else if (nullptr == queryText->completer() && m_completer) {
+        queryText->setCompleter(m_completer);
+    }
     return false;
 }
 
-// onCompletionActivated() is called when an entry is selected in the
-// popup, but the edit text is going to be replaced in any case if
-// there is a current match (we can't prevent it in the signal). If
-// there is no match (e.g. the user clicked the history button and
-// selected an entry), the query text will not be set.
-// So:
+// onCompletionActivated() is called when an entry is selected in the popup, but the edit text is
+// going to be replaced in any case if there is a current match (we can't prevent it in the
+// signal). If there is no match (e.g. the user clicked the history button and selected an entry),
+// the query text will not be set.  So:
 //  - We set the query text to the popup activation value in all cases
-//  - We schedule a callback to set the text to what we want (which is the
-//    concatenation of the user entry before the current partial word and the
-//    pop up data.
-//  - Note that a history click will replace a current partial word,
-//    so that the effect is different if there is a space at the end
-//    of the entry or not: pure concatenation vs replacement of the
+//  - We schedule a callback to set the text to what we want (which is the concatenation of the user
+//    entry before the current partial word and the pop up data.
+//  - Note that a history click will replace a current partial word, so that the effect is different
+//    if there is a space at the end of the entry or not: pure concatenation vs replacement of the
 //    last (partial) word.
-void SSearch::restoreText()
+void SSearch::restoreText(bool startsearch)
 {
-    LOGDEB("SSearch::restoreText: savedEdit: " << qs2u8s(m_savedEditText) <<
-           endl);
+    LOGDEB("SSearch::restoreText: ss " << startsearch << " savedEdit: " <<
+           qs2u8s(m_savedEditText) << '\n');
+
     if (!m_savedEditText.trimmed().isEmpty()) {
         // If the popup text begins with the saved text, just let it replace
         if (currentText().lastIndexOf(m_savedEditText) != 0) {
@@ -348,10 +394,11 @@ void SSearch::restoreText()
         m_savedEditText = "";
     }        
     queryText->setFocus();
-    if (prefs.ssearchStartOnComplete) {
+    if (startsearch || prefs.ssearchStartOnComplete) {
         QTimer::singleShot(0, this, SLOT(startSimpleSearch()));
     }
 }
+
 void SSearch::onCompletionActivated(const QString& text)
 {
     LOGDEB("SSearch::onCompletionActivated: queryText [" <<
@@ -369,13 +416,14 @@ void SSearch::onHistoryClicked()
     }
 }
 
+// Connected to queryText::textEdited
 void SSearch::searchTextEdited(const QString& text)
 {
     LOGDEB1("SSearch::searchTextEdited: text [" << qs2u8s(text) << "]\n");
     QString pword;
-    int cs = getPartialWord(pword);
-    int tp = searchTypCMB->currentIndex();
-    
+    int cs = getPartialWord(currentText(), pword);
+    int tp = getSearchType();
+
     m_savedEditText = text.left(cs);
     LOGDEB1("SSearch::searchTextEdited: cs " <<cs<<" pword ["<< qs2u8s(pword) <<
             "] savedEditText [" << qs2u8s(m_savedEditText) << "]\n");
@@ -403,18 +451,17 @@ void SSearch::searchTextChanged(const QString& text)
 
 void SSearch::onSearchTypeChanged(int typ)
 {
-    LOGDEB1("Search type now " << typ << "\n");
+    LOGDEB1("SSearch::onSearchTypeChanged: type now " << typ << "\n");
 
-    // This may come from the menus or the combobox. Ensure that
-    // things are in sync. No loop because we are connected to
-    // combobox or menu activated(), not currentIndexChanged()
+    // This may come from the menus or the combobox. Ensure that things are in sync. No loop because
+    // we are connected to combobox or menu activated(), not currentIndexChanged()
     searchTypCMB->setCurrentIndex(typ);
     
     // Adjust context help
     if (typ == SST_LANG) {
-        HelpClient::installMap((const char *)this->objectName().toUtf8(), "RCL.SEARCH.LANG");
+        HelpClient::installMap(qs2u8s(objectName()), "RCL.SEARCH.LANG");
     } else {
-        HelpClient::installMap((const char *)this->objectName().toUtf8(), "RCL.SEARCH.GUI.SIMPLE");
+        HelpClient::installMap(qs2u8s(objectName()), "RCL.SEARCH.GUI.SIMPLE");
     }
     // Also fix tooltips
     switch (typ) {
@@ -476,7 +523,7 @@ void SSearch::startSimpleSearch()
         && !queryText->completer()->currentCompletion().isEmpty()) {
         return;
     }
-    string u8 = qs2u8s(queryText->text());
+    auto u8 = qs2u8s(queryText->text());
     trimstring(u8);
     if (u8.length() == 0)
         return;
@@ -494,8 +541,7 @@ void SSearch::startSimpleSearch()
         prefs.ssearchHistory.removeDuplicates();
     }
     if (prefs.historysize >= 0) {
-        for (int i = (int)prefs.ssearchHistory.count();
-             i > prefs.historysize; i--) {
+        for (int i = (int)prefs.ssearchHistory.count(); i > prefs.historysize; i--) {
             prefs.ssearchHistory.removeLast();
         }
     }
@@ -505,30 +551,29 @@ void SSearch::setPrefs()
 {
 }
 
-string SSearch::asXML()
+std::string SSearch::asXML()
 {
     return m_xml;
 }
 
-bool SSearch::startSimpleSearch(const string& u8, int maxexp)
+bool SSearch::startSimpleSearch(const std::string& u8, int maxexp)
 {
     LOGDEB("SSearch::startSimpleSearch(" << u8 << ")\n");
-    string stemlang = prefs.stemlang();
+    auto stemlang = prefs.stemlang();
 
-    ostringstream xml;
+    std::ostringstream xml;
     xml << "<SD type='ssearch'>\n";
     xml << "  <SL>" << stemlang << "</SL>\n";
     xml << "  <T>" << base64_encode(u8) << "</T>\n";
 
-    SSearchType tp = (SSearchType)searchTypCMB->currentIndex();
+    auto tp = getSearchType();
     std::shared_ptr<Rcl::SearchData> sdata;
 
     if (tp == SST_LANG) {
         xml << "  <SM>QL</SM>\n";
-        string reason;
+        std::string reason;
         if (prefs.autoSuffsEnable) {
-            sdata = wasaStringToRcl(theconfig, stemlang, u8, reason, 
-                                    (const char *)prefs.autoSuffs.toUtf8());
+            sdata = wasaStringToRcl(theconfig, stemlang, u8, reason, qs2u8s(prefs.autoSuffs));
             if (!prefs.autoSuffs.isEmpty()) {
                 xml <<  "  <AS>" << qs2u8s(prefs.autoSuffs) << "</AS>\n";
             }
@@ -536,8 +581,7 @@ bool SSearch::startSimpleSearch(const string& u8, int maxexp)
             sdata = wasaStringToRcl(theconfig, stemlang, u8, reason);
         }
         if (!sdata) {
-            QMessageBox::warning(0, "Recoll", tr("Bad query string") + ": " +
-                                 QString::fromUtf8(reason.c_str()));
+            QMessageBox::warning(0, "Recoll", tr("Bad query string") + ": " + u8s2qs(reason));
             return false;
         }
     } else {
@@ -551,6 +595,10 @@ bool SSearch::startSimpleSearch(const string& u8, int maxexp)
             xml << "  <SM>FN</SM>\n";
             clp = new Rcl::SearchDataClauseFilename(u8);
         } else {
+            // AND/OR modes have no subsearches so it's ok to set the no wild cards flag on the top
+            // and only SearchData object
+            if (prefs.ignwilds)
+                sdata->setNoWildExp(true);
             // ANY or ALL, several words.
             if (tp == SST_ANY) {
                 xml << "  <SM>OR</SM>\n";
@@ -589,8 +637,7 @@ bool SSearch::checkExtIndexes(const std::vector<std::string>& dbs)
 {
     std::string reason;
     if (!maybeOpenDb(reason, false)) {
-        QMessageBox::critical(0, "Recoll", tr("Can't open index") +
-                              u8s2qs(reason));
+        QMessageBox::critical(0, "Recoll", tr("Can't open index") + u8s2qs(reason));
         return false;
     }
     if (!rcldb->setExtraQueryDbs(dbs)) {
@@ -601,15 +648,15 @@ bool SSearch::checkExtIndexes(const std::vector<std::string>& dbs)
 
 bool SSearch::fromXML(const SSearchDef& fxml)
 {
-    string asString;
-    set<string> cur;
-    set<string> stored;
+    std::string asString;
+    std::set<std::string> cur;
+    std::set<std::string> stored;
 
     // Retrieve current list of stemlangs. prefs returns a
     // space-separated list Warn if stored differs from current,
     // but don't change the latter.
     stringToStrings(prefs.stemlang(), cur);
-    stored = set<string>(fxml.stemlangs.begin(), fxml.stemlangs.end());
+    stored = std::set<std::string>(fxml.stemlangs.begin(), fxml.stemlangs.end());
     stringsToString(fxml.stemlangs, asString);
     if (cur != stored) {
         QMessageBox::warning(
@@ -620,7 +667,7 @@ bool SSearch::fromXML(const SSearchDef& fxml)
 
     // Same for autosuffs
     stringToStrings(qs2u8s(prefs.autoSuffs), cur);
-    stored = set<string>(fxml.autosuffs.begin(), fxml.autosuffs.end());
+    stored = std::set<std::string>(fxml.autosuffs.begin(), fxml.autosuffs.end());
     stringsToString(fxml.stemlangs, asString);
     if (cur != stored) {
         QMessageBox::warning(
@@ -629,7 +676,6 @@ bool SSearch::fromXML(const SSearchDef& fxml)
             tr(" differ from current preferences (kept)"));
     }
 
-
     if (!checkExtIndexes(fxml.extindexes)) {
         stringsToString(fxml.extindexes, asString);
         QMessageBox::warning(
@@ -637,7 +683,7 @@ bool SSearch::fromXML(const SSearchDef& fxml)
             tr("Could not restore external indexes for stored query:<br> ") +
             (rcldb ? u8s2qs(rcldb->getReason()) : tr("???")) + QString("<br>") +
             tr("Using current preferences."));
-        string s;
+        std::string s;
         maybeOpenDb(s, true);
     } else {
         prefs.useTmpActiveExtraDbs = true;
@@ -671,38 +717,34 @@ bool SSearch::hasSearchString()
     return !currentText().isEmpty();
 }
 
-// Add term to simple search. Term comes out of double-click in
-// reslist or preview. 
-// It would probably be better to cleanup in preview.ui.h and
-// reslist.cpp and do the proper html stuff in the latter case
-// (which is different because it format is explicit richtext
-// instead of auto as for preview, needed because it's built by
-// fragments?).
+// Add term to simple search. Term comes out of double-click in reslist or preview.
+// It would probably be better to cleanup in preview.ui.h and reslist.cpp and do the proper html
+// stuff in the latter case (which is different because it format is explicit richtext instead of
+// auto as for preview, needed because it's built by fragments?).
 static const char* punct = " \t()<>\"'[]{}!^*.,:;\n\r";
 void SSearch::addTerm(QString term)
 {
-    LOGDEB("SSearch::AddTerm: [" << qs2u8s(term) << "]\n");
-    string t = (const char *)term.toUtf8();
-    string::size_type pos = t.find_last_not_of(punct);
-    if (pos == string::npos)
+    auto t = qs2u8s(term);
+    LOGDEB("SSearch::AddTerm: [" << t << "]\n");
+    std::string::size_type pos = t.find_last_not_of(punct);
+    if (pos == std::string::npos)
         return;
     t = t.substr(0, pos+1);
     pos = t.find_first_not_of(punct);
-    if (pos != string::npos)
+    if (pos != std::string::npos)
         t = t.substr(pos);
     if (t.empty())
         return;
-    term = QString::fromUtf8(t.c_str());
+    term = u8s2qs(t);
 
     QString text = currentText();
-    text += QString::fromLatin1(" ") + term;
+    text += " " + term;
     queryText->setText(text);
 }
 
 void SSearch::onWordReplace(const QString& o, const QString& n)
 {
-    LOGDEB("SSearch::onWordReplace: o [" << qs2u8s(o) << "] n [" <<
-           qs2u8s(n) << "]\n");
+    LOGDEB("SSearch::onWordReplace: o [" << qs2u8s(o) << "] n [" << qs2u8s(n) << "]\n");
     QString txt = currentText();
     QRegularExpression exp(QString("\\b") + o + QString("\\b"),
                            QRegularExpression::CaseInsensitiveOption);
@@ -717,42 +759,3 @@ void SSearch::setAnyTermMode()
 {
     searchTypCMB->setCurrentIndex(SST_ANY);
 }
-
-// If text does not end with space, return last (partial) word and >0
-// else return -1
-int SSearch::getPartialWord(QString& word)
-{
-    // Extract last word in text
-    QString txt = currentText();
-    if (txt.isEmpty()) {
-        return -1;
-    }
-    int lstidx = txt.size()-1;
-
-    // If the input ends with a space or dquote (phrase input), or
-    // dquote+qualifiers, no partial word.
-    if (txt[lstidx] == ' ') {
-        return -1;
-    }
-    int cs = txt.lastIndexOf("\"");
-    if (cs > 0) {
-        bool dquoteToEndNoSpace{true};
-        for (int i = cs; i <= lstidx; i++) {
-            if (txt[i] == ' ') {
-                dquoteToEndNoSpace = false;
-                break;
-            }
-        }
-        if (dquoteToEndNoSpace) {
-            return -1;
-        }
-    }
-
-    cs = txt.lastIndexOf(" ");
-    if (cs < 0)
-        cs = 0;
-    else
-        cs++;
-    word = txt.right(txt.size() - cs);
-    return cs;
-}
